@@ -8,11 +8,13 @@ Enhancements by James Stewart since 2021
 Library for communicating with the BES (BigFix) REST API.
 """
 
+import datetime
 import json
+import logging
 import os
+import random
 import site
 import string
-import sys
 
 # import urllib3.poolmanager
 
@@ -25,6 +27,19 @@ import requests
 from lxml import etree, objectify
 from pkg_resources import resource_filename
 
+logging.basicConfig(level=logging.WARNING)
+besapi_logger = logging.getLogger("besapi")
+
+
+def rand_password(length=20):
+    """get a random password"""
+
+    all_safe_chars = string.ascii_letters + string.digits + "!#()*+,-.:;<=>?[]^_|~"
+
+    # https://medium.com/analytics-vidhya/create-a-random-password-generator-using-python-2fea485e9da9
+    password = "".join(random.sample(all_safe_chars, length))
+    return password
+
 
 def sanitize_txt(*args):
     """Clean arbitrary text for safe file system usage."""
@@ -35,7 +50,7 @@ def sanitize_txt(*args):
         sani_args.append(
             "".join(
                 c
-                for c in str(arg).replace("/", "-").replace("\\", "-")
+                for c in str(arg).replace("/", "-").replace("\\", "-").replace(" ", "_")
                 if c in valid_chars
             )
             .encode("ascii", "ignore")
@@ -122,6 +137,11 @@ class BESConnection:
         self.username = username
         self.session = requests.Session()
         self.session.auth = (username, password)
+        # store info on operator used to login
+        # self.operator_info = {}
+
+        # use a sitepath context if none specified when required.
+        self.site_path = "master"
 
         # if not provided, add on https://
         if not rootserver.startswith("http"):
@@ -132,6 +152,7 @@ class BESConnection:
 
         self.rootserver = rootserver
         self.verify = verify
+        self.last_connected = None
 
         self.login()
 
@@ -169,30 +190,42 @@ class BESConnection:
 
     def get(self, path="help", **kwargs):
         """HTTP GET request"""
+        # avoid infinite loop:
+        if "login" not in path:
+            self.login()
+        self.last_connected = datetime.datetime.now()
         return RESTResult(
             self.session.get(self.url(path), verify=self.verify, **kwargs)
         )
 
     def post(self, path, data, **kwargs):
         """HTTP POST request"""
+        self.login()
+        self.last_connected = datetime.datetime.now()
         return RESTResult(
             self.session.post(self.url(path), data=data, verify=self.verify, **kwargs)
         )
 
     def put(self, path, data, **kwargs):
         """HTTP PUT request"""
+        self.login()
+        self.last_connected = datetime.datetime.now()
         return RESTResult(
             self.session.put(self.url(path), data=data, verify=self.verify, **kwargs)
         )
 
     def delete(self, path, **kwargs):
         """HTTP DELETE request"""
+        self.login()
+        self.last_connected = datetime.datetime.now()
         return RESTResult(
             self.session.delete(self.url(path), verify=self.verify, **kwargs)
         )
 
     def session_relevance_xml(self, relevance, **kwargs):
         """Get Session Relevance Results XML"""
+        self.login()
+        self.last_connected = datetime.datetime.now()
         return RESTResult(
             self.session.post(
                 self.url("query"),
@@ -226,33 +259,185 @@ class BESConnection:
         rel_result_array = self.session_relevance_array(relevance, **kwargs)
         return "\n".join(rel_result_array)
 
-    def connected(self):
-        """return true if connected"""
-        return bool(self.get("login").request.status_code == 200)
-
     def login(self):
         """do login"""
-        if not self.connected():
-            self.get("login").request.raise_for_status()
+        if bool(self.last_connected):
+            duration_obj = datetime.datetime.now() - self.last_connected
+            duration_minutes = duration_obj / datetime.timedelta(minutes=1)
+            besapi_logger.info(
+                "Connection Time: `%s` - Duration: %d minutes",
+                self.last_connected,
+                duration_minutes,
+            )
+            if int(duration_minutes) > 3:
+                besapi_logger.info("Refreshing Login to prevent timeout.")
+                self.last_connected = None
+
+        if not bool(self.last_connected):
+            result_login = self.get("login")
+            if not result_login.request.status_code == 200:
+                result_login.request.raise_for_status()
+            if result_login.request.status_code == 200:
+                # set time of connection
+                self.last_connected = datetime.datetime.now()
 
         # This doesn't work until urllib3 is updated to a future version:
         # if self.connected():
         #     self.session.mount(self.url("upload"), HTTPAdapterBiggerBlocksize())
 
-        return self.connected()
+        return bool(self.last_connected)
 
     def logout(self):
         """clear session and close it"""
         self.session.cookies.clear()
         self.session.close()
 
+    def validate_site_path(self, site_path, check_site_exists=True, raise_error=False):
+        """make sure site_path is valid"""
+
+        if site_path is None:
+            if not raise_error:
+                return None
+            raise ValueError("Site Path is `None` - NoneType Error")
+        if str(site_path).strip() == "":
+            if not raise_error:
+                return None
+            raise ValueError("Site Path is empty!")
+
+        # options for valid site prefix: (master must be last, special case)
+        site_prefixes = ["external/", "custom/", "operator/", "master"]
+
+        for prefix in site_prefixes:
+            if prefix in site_path:
+                if prefix == "master" and prefix != site_path:
+                    # Invalid: This error should be raised regardless
+                    raise ValueError(
+                        f"Site path for master actionsite must be `master` not `{site_path}`"
+                    )
+                if not check_site_exists:
+                    # don't check if site exists first
+                    return site_path
+                else:
+                    # check site exists first
+                    site_result = self.get(f"site/{site_path}")
+                    if site_result.request.status_code != 200:
+                        besapi_logger.info("Site `%s` does not exist", site_path)
+                        if not raise_error:
+                            return None
+
+                        raise ValueError(f"Site at path `{site_path}` does not exist!")
+
+                    # site_path is valid and exists:
+                    return site_path
+
+        # Invalid: No valid prefix found
+        raise ValueError(
+            f"Site Path does not start with a valid prefix! {site_prefixes}"
+        )
+
+    def get_current_site_path(self, site_path=None):
+        """if site_path is none, get current instance site_path,
+        otherwise validate and return provided site_path"""
+
+        # use instance site_path context if none provided:
+        if site_path is None or str(site_path).strip() == "":
+            site_path = self.site_path
+
+        if site_path is None or str(site_path).strip() == "":
+            besapi_logger.error("Site Path context not set and Site Path not provided!")
+            raise ValueError("Site Path context not set and Site Path not provided!")
+
+        # don't check for site's existence when doing basic get
+        return self.validate_site_path(site_path, check_site_exists=False)
+
+    def set_current_site_path(self, site_path):
+        """set current site path context"""
+
+        if self.validate_site_path(site_path):
+            self.site_path = site_path
+            return self.site_path
+
+    def create_site_from_file(self, bes_file_path, site_type="custom"):
+        """create new site"""
+        xml_parsed = etree.parse(bes_file_path)
+        new_site_name = xml_parsed.xpath("/BES/CustomSite/Name/text()")[0]
+
+        result_site_path = self.validate_site_path(
+            site_type + "/" + new_site_name, True, False
+        )
+
+        if result_site_path:
+            besapi_logger.warning("Site `%s` already exists", result_site_path)
+            return None
+
+        result_site = self.post("sites", etree.tostring(xml_parsed))
+
+        return result_site
+
+    def get_user(self, user_name):
+        """get a user"""
+
+        result_users = self.get(f"operator/{user_name}")
+
+        if result_users and "Operator does not exist" not in str(result_users):
+            return result_users
+
+        besapi_logger.info("User `%s` Not Found!", user_name)
+
+    def create_user_from_file(self, bes_file_path):
+        """create user from xml"""
+        xml_parsed = etree.parse(bes_file_path)
+        new_user_name = xml_parsed.xpath("/BESAPI/Operator/Name/text()")[0]
+        result_user = self.get_user(new_user_name)
+
+        if result_user:
+            besapi_logger.warning("User `%s` Already Exists!", new_user_name)
+            return result_user
+        besapi_logger.info("Creating User `%s`", new_user_name)
+        _ = self.post("operators", etree.tostring(xml_parsed))
+        # print(user_result)
+        return self.get_user(new_user_name)
+
+    def get_computergroup(self, group_name, site_path=None):
+        """get computer group resource URI"""
+
+        site_path = self.get_current_site_path(site_path)
+        result_groups = self.get(f"computergroups/{site_path}")
+
+        for group in result_groups.besobj.ComputerGroup:
+            if group_name == str(group.Name):
+                besapi_logger.info(
+                    "Found Group With Resource: %s", group.attrib["Resource"]
+                )
+                return group
+
+        besapi_logger.info("Group `%s` Not Found!", group_name)
+
+    def create_group_from_file(self, bes_file_path, site_path=None):
+        """create a new group"""
+        site_path = self.get_current_site_path(site_path)
+        xml_parsed = etree.parse(bes_file_path)
+        new_group_name = xml_parsed.xpath("/BES/ComputerGroup/Title/text()")[0]
+
+        existing_group = self.get_computergroup(new_group_name, site_path)
+
+        if existing_group is not None:
+            besapi_logger.warning("Group `%s` Already Exists!", new_group_name)
+            return existing_group
+
+        # print(lxml.etree.tostring(xml_parsed))
+
+        _ = self.post(f"computergroups/{site_path}", etree.tostring(xml_parsed))
+
+        return self.get_computergroup(site_path, new_group_name)
+
     def upload(self, file_path, file_name=None):
         """
         upload a single file
         https://developer.bigfix.com/rest-api/api/upload.html
         """
         if not os.access(file_path, os.R_OK):
-            print(file_path, "is not readable")
+            besapi_logger.error(file_path, "is not readable")
             raise FileNotFoundError
 
         # if file_name not specified, then get it from tail of file_path
@@ -264,14 +449,39 @@ class BESConnection:
         with open(file_path, "rb") as f:
             return self.post(self.url("upload"), data=f, headers=headers)
 
+    def get_content_by_resource(self, resource_url):
+        """get a single content item by resource"""
+        # Get Specific Content
+        content = None
+        try:
+            content = self.get(resource_url.replace("http://", "https://"))
+        except PermissionError as err:
+            logging.error("Could not export item:")
+            logging.error(err)
+
+        # item_id = int(resource_url.split("/")[-1])
+        # site_name = resource_url.split("/")[-2]
+        # if site_name == "master":
+        #     site_path = site_name
+        # else:
+        #     site_path = resource_url.split("/")[-3] + "/" + site_name
+        return content
+
     def export_site_contents(
-        self, site_path, export_folder="./", name_trim=70, verbose=False
+        self,
+        site_path=None,
+        export_folder="./",
+        name_trim=100,
+        verbose=False,
+        include_site_folder=True,
+        include_item_ids=True,
     ):
         """export contents of site
         Originally here:
         - https://gist.github.com/jgstew/1b2da12af59b71c9f88a
         - https://bigfix.me/fixlet/details/21282
         """
+        site_path = self.get_current_site_path(site_path)
         if verbose:
             print("export_site_contents()")
         # Iterate Over All Site Content
@@ -297,34 +507,48 @@ class BESConnection:
                     )
 
                 # Get Specific Content
-                content = self.get(
-                    item.attrib["Resource"].replace("http://", "https://")
-                )
+                content = self.get_content_by_resource(item.attrib["Resource"])
+
+                if not content:
+                    continue
 
                 # Write Content to Disk
-                if content:
-                    if not os.path.exists(
-                        export_folder + "%s/%s" % sanitize_txt(site_path, item.tag)
-                    ):
-                        os.makedirs(
-                            export_folder + "%s/%s" % sanitize_txt(site_path, item.tag)
-                        )
+                item_folder = export_folder + "%s/%s" % sanitize_txt(
+                    site_path, item.tag
+                )
+                if not include_site_folder:
+                    item_folder = export_folder + "%s" % sanitize_txt(item.tag)
+                if not os.path.exists(item_folder):
+                    os.makedirs(item_folder)
 
-                    with open(
-                        export_folder
-                        + "%s/%s/%s - %s.bes"
-                        % sanitize_txt(
-                            site_path,
+                item_path = export_folder + "%s/%s/%s-%s.bes" % sanitize_txt(
+                    site_path,
+                    item.tag,
+                    item.ID,
+                    item.Name.text[:name_trim],
+                )
+                if not include_item_ids:
+                    item_path = export_folder + "%s/%s/%s.bes" % sanitize_txt(
+                        site_path,
+                        item.tag,
+                        item.Name.text[:name_trim],
+                    )
+                if not include_site_folder:
+                    item_path = export_folder + "%s/%s-%s.bes" % sanitize_txt(
+                        item.tag,
+                        item.ID,
+                        item.Name.text[:name_trim],
+                    )
+                    if not include_item_ids:
+                        item_path = export_folder + "%s/%s.bes" % sanitize_txt(
                             item.tag,
-                            item.ID,
-                            # http://stackoverflow.com/questions/2872512/python-truncate-a-long-string
-                            # trimming to 150 worked in most cases, but recently even that had issues.
-                            # now trimmed to first name_trim characters of the title of the item.
                             item.Name.text[:name_trim],
-                        ),
-                        "wb",
-                    ) as bes_file:
-                        bes_file.write(content.text.encode("utf-8"))
+                        )
+                with open(
+                    item_path,
+                    "wb",
+                ) as bes_file:
+                    bes_file.write(content.text.encode("utf-8"))
 
     def export_all_sites(
         self, include_external=False, export_folder="./", name_trim=70, verbose=False
@@ -345,7 +569,6 @@ class BESConnection:
     __call__ = login
     # https://stackoverflow.com/q/40536821/861745
     __enter__ = login
-    __exit__ = logout
 
 
 class RESTResult:
@@ -359,6 +582,22 @@ class RESTResult:
         self._besdict = None
         self._besjson = None
 
+        try:
+            if self.request.status_code == 403:
+                # Error most likely due to not having master operator privs
+                # Could also be due to non-master operator not having specific privs
+                raise PermissionError(
+                    f"\n - HTTP Response Status Code: `403` Forbidden\n - ERROR: `{self.text}`\n - URL: `{self.request.url}`"
+                )
+
+            besapi_logger.info(
+                "HTTP Request Status Code `%d` from URL `%s`",
+                self.request.status_code,
+                self.request.url,
+            )
+        except AttributeError as err:
+            besapi_logger.warning("Error (expected during tests) %s", err)
+
         if (
             "content-type" in request.headers
             and request.headers["content-type"] == "application/xml"
@@ -438,8 +677,8 @@ class RESTResult:
                 xmlschema = etree.XMLSchema(xmlschema_doc)
             except etree.XMLSchemaParseError as err:
                 # this should only error if the XSD itself is malformed
-                print(f"ERROR with {xsd}: {err}")
-                raise
+                besapi_logger.error("ERROR with `%s`: %s", xsd, err)
+                raise err
 
             if xmlschema.validate(xmldoc):
                 return True
@@ -470,7 +709,7 @@ def main():
     # pylint: disable=import-outside-toplevel
     try:
         from bescli import bescli
-    except (ImportError, ModuleNotFoundError):
+    except ImportError:
         site.addsitedir(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
         from bescli import bescli
     bescli.main()
