@@ -8,17 +8,14 @@ Simple command line interface for the BES (BigFix) REST API.
 """
 
 import getpass
+import logging
 import os
 import site
+from configparser import ConfigParser as SafeConfigParser
 
 import requests.exceptions
 from cmd2 import Cmd
 
-try:
-    from ConfigParser import SafeConfigParser
-except (ImportError, ModuleNotFoundError):
-    from configparser import SafeConfigParser
-
 try:
     from besapi import besapi
 except ModuleNotFoundError:
@@ -163,7 +160,7 @@ class BESCLInterface(Cmd):
         if self.BES_USER_NAME and self.BES_ROOT_SERVER and self.BES_PASSWORD:
             try:
                 self.bes_conn = besapi.BESConnection(
-                    user, self.BES_PASSWORD, root_server
+                    self.BES_USER_NAME, self.BES_PASSWORD, self.BES_ROOT_SERVER
                 )
                 if self.bes_conn.login():
                     self.pfeedback("Login Successful!")
@@ -202,7 +199,8 @@ class BESCLInterface(Cmd):
         """Logout and clear session"""
         if self.bes_conn:
             self.bes_conn.logout()
-            self.bes_conn = None
+            # del self.bes_conn
+            # self.bes_conn = None
         self.pfeedback("Logout Complete!")
 
     def do_debug(self, setting):
@@ -212,12 +210,16 @@ class BESCLInterface(Cmd):
         self.echo = bool(setting)
         self.quiet = bool(setting)
         self.timing = bool(setting)
+        if bool(setting):
+            logging.getLogger("besapi").setLevel(logging.DEBUG)
+        else:
+            logging.getLogger("besapi").setLevel(logging.WARNING)
 
     def do_clear(self, arg=None):
         """clear current config and logout"""
         if self.bes_conn:
             self.bes_conn.logout()
-            self.bes_conn = None
+            # self.bes_conn = None
         if arg and "root" in arg.lower():
             self.pfeedback(" - clearing root server parameter -")
             self.BES_ROOT_SERVER = None
@@ -255,16 +257,21 @@ class BESCLInterface(Cmd):
 
     def do_ls(self, arg=None):
         """List the current settings and connection status"""
+        print("        Connected: " + str(bool(self.bes_conn)))
         print(
-            "BES_ROOT_SERVER: " + (self.BES_ROOT_SERVER if self.BES_ROOT_SERVER else "")
+            "  BES_ROOT_SERVER: "
+            + (self.BES_ROOT_SERVER if self.BES_ROOT_SERVER else "")
         )
-        print("  BES_USER_NAME: " + (self.BES_USER_NAME if self.BES_USER_NAME else ""))
         print(
-            "Password Length: "
+            "    BES_USER_NAME: " + (self.BES_USER_NAME if self.BES_USER_NAME else "")
+        )
+        print(
+            "  Password Length: "
             + str(len(self.BES_PASSWORD if self.BES_PASSWORD else ""))
         )
-        print("      Connected: " + str(bool(self.bes_conn)))
-        print("Config File Path: " + self.conf_path)
+        print(" Config File Path: " + self.conf_path)
+        if self.bes_conn:
+            print("Current Site Path: " + self.bes_conn.get_current_site_path(None))
 
     def do_error_count(self, arg=None):
         """Output the number of errors"""
@@ -294,9 +301,27 @@ class BESCLInterface(Cmd):
         """output version of besapi"""
         self.poutput(f"besapi version: {__version__}")
 
+    def do_get_current_site(self, statement=None):
+        """output current site path context"""
+        self.poutput(
+            f"Current Site Path: `{ self.bes_conn.get_current_site_path(None) }`"
+        )
+
+    def do_set_current_site(self, statement=None):
+        """set current site path context"""
+        self.poutput(
+            f"New Site Path: `{ self.bes_conn.set_current_site_path(statement) }`"
+        )
+
+    def do_get_content(self, resource_url):
+        """get a specific item by resource url"""
+        print(self.bes_conn.get_content_by_resource(resource_url))
+
     def do_export_site(self, site_path):
         """export site contents to current folder"""
-        self.bes_conn.export_site_contents(site_path, verbose=True)
+        self.bes_conn.export_site_contents(
+            site_path, verbose=True, include_site_folder=False, include_item_ids=False
+        )
 
     def do_export_all_sites(self, statement=None):
         """export site contents to current folder"""
@@ -311,6 +336,33 @@ class BESCLInterface(Cmd):
         else:
             print(self.bes_conn.upload(file_path))
 
+    complete_create_group = Cmd.path_complete
+
+    def do_create_group(self, file_path):
+        """create bigfix group from bes file"""
+        if not os.access(file_path, os.R_OK):
+            print(file_path, "is not a readable file")
+        else:
+            print(self.bes_conn.create_group_from_file(file_path))
+
+    complete_create_user = Cmd.path_complete
+
+    def do_create_user(self, file_path):
+        """create bigfix user from bes file"""
+        if not os.access(file_path, os.R_OK):
+            print(file_path, "is not a readable file")
+        else:
+            print(self.bes_conn.create_user_from_file(file_path))
+
+    complete_create_site = Cmd.path_complete
+
+    def do_create_site(self, file_path):
+        """create bigfix site from bes file"""
+        if not os.access(file_path, os.R_OK):
+            print(file_path, "is not a readable file")
+        else:
+            print(self.bes_conn.create_site_from_file(file_path))
+
 
 def main():
     """Run the command loop if invoked"""
